#!/usr/bin/env python
''' abyss.player '''

import os
from abyss.db import stored, DB, where, deserialize
from pbkdf2 import PBKDF2

class User(stored):
    __slots__ = ('username', 'hashed_password', 'salt')
    table = DB.table('User')

    def __init__(self, username=None, hashed_password=None, salt=None,
            **kwargs):
        self.username = username
        self.hashed_password = hashed_password
        self.salt = salt

    def for_client(self):
        return {'username': self.username}

    @classmethod
    def from_server(cls, username=None):
        return User(username=username)

    @classmethod
    def get_userdata(cls, username):
        return cls.table.get(where('username') == username)

    @classmethod
    def get_user(cls, username):
        userdata = cls.get_userdata(username)
        if userdata is None:
            return None
        return deserialize(userdata)

    @classmethod
    def authenticate(cls, username, password):
        user = cls.get_user(username)
        if user is None:
            return None
        hashed_password = PBKDF2(
                password, user.salt.decode('base64')).read(32)
        if user.hashed_password.decode('base64') == hashed_password:
            return user
        return None

    @classmethod
    def new(cls, username, password):
        userdata = cls.get_userdata(username)
        if userdata:
            return None
        salt = os.urandom(8)
        hashed_password = PBKDF2(password, salt).read(32)
        user = User(
            username=username, hashed_password=hashed_password.encode('base64'),
            salt=salt.encode('base64')
        )
        user.store()
        return user



#!/usr/bin/env python
''' abyss.server.mapstate '''
import json
from random import uniform
from abyss.entity import Manapool

class MapState(object):

    def __init__(self):
        pass

    def deserialize(self, data):
        self.map = data['map']
        self.height = data['height']
        self.width = data['width']

    def serialize(self):
        return {'map': self.map, 'height': self.height, 'width': self.width}

    def generate_new(self, width=100, height=100, **kwargs):
        self.map = [[None]*width for h in range(height)]
        self.width = width
        self.height = height
        self.spawn_manapools()

    def spawn_manapools(self, num=10):
        for i in range(num):
            x = int(uniform(0, self.width))
            y = int(uniform(0, self.height))
            self.map[y][x] = Manapool()



#!/usr/bin/env python
''' abyss.entity '''
from abyss.account import User

class owned(stored):
    
    def serialize(self):
        d = {}
        for attr in self.__slots__:
            if attr == 'owner':
                val = self.owner.username if self.owner else None
            else:
                val = getattr(self, attr)
            d[attr] = val
        return d

    @classmethod
    def deserialize(cls, **kwargs):
        kwargs_fixed = {}
        for k, v in kwargs.items():
            if k == 'owner':
                kwargs_fixed[k] = User.get_user(v)
            else:
                kwargs_fixed[k] = v

class Entity(owned):
    pass

class Manapool(Entity):
    __slots__ = ('owner', 'managen', 'mana')

    def __init__(self, owner=None, managen=1.0, **kwargs):
        self.owner = owner
        self.managen = managen
        self.mana = 0.0



#!/usr/bin/env python
import requests
from functools import wraps
from abyss.server import route
from abyss.account import User


BASE_URL = 'http://localhost:8080'

class Client(object):

    def __init__(self, username=None, password=None, base_url=BASE_URL):
        self.auth = (username, password)
        self.url = base_url

    def get_home(self):
        rt = self.url + route('base')
        return requests.get(rt, auth=self.auth).json()

    def get_user(self, username):
        rt = self.url + route('user').format(username=username)
        response = requests.get(rt, auth=self.auth)
        return User.from_server(**response.json())
    
def main():
    import argparse
    parser = argparse.ArgumentParser(prog='abyss')
    parser.add_argument('--username', '-u')
    parser.add_argument('--password', '-p')
    args = parser.parse_args()
    client = Client(username=args.username, password=args.password)
    print client.get_home()
    print client.get_user('foo').username

if __name__ == '__main__':
    main()



#!/usr/bin/env python

import os
import json
from functools import wraps

from flask import Flask, request

from abyss.server import Route
from abyss.account import User

app = Flask('abyss')

def check_auth(auth):
    if not auth:
        return None
    return User.authenticate(auth['username'], auth['password']) 

def routed(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        auth = request.authorization
        kwargs['authuser'] = check_auth(auth)
        ret = func(*args, **kwargs)
        return json.dumps(ret)
    return wrapper

@app.route(Route.BASE)
@routed
def index(authuser=None):
    return {'username': authuser.username if authuser else None}

@app.route(Route.USER)
@routed
def user(username, authuser=None):
    user = User.get_user(username)
    if user:
        user = user.for_client()
    return user

def main():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--host', '-H', default='127.0.0.1')
    parser.add_argument('--port', '-p', type=int, default=8080)
    parser.add_argument('--debug', '-d', action='store_true')
    args = parser.parse_args()
    app.run(host=args.host, port=args.port, debug=args.debug)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
''' abyss.server.gamestate
Holds, loads and dumps the gamestate
'''
import json
from abyss.server.mapstate import MapState

class GameState(object):
    
    def __init__(self, path):
        self.path = path
        self.map = MapState()

    def load(self):
        with open(self.path) as f:
            data = json.load(f)
        self.deserialize(data)

    def new(self):
        self.generate_new()
        self.dump(self.path)

    def deserialize(self, data):
        self.map.deserialize(data['map'])

    def serialize(self):
        return {'map': self.map.serialize()}

    def generate_new(self, **kwargs):
        self.map.generate_new(**kwargs)

    def dump(self, path):
        data = self.serialize()
        with open(path, 'w') as f:
            json.dump(data, f, indent=4)



#!/usr/bin/env python
''' abyss.server
Abyss HTTP API server
'''

class Route(object):
    BASE = '/'
    USER = '/user/<username>'

def route(attr):
    val = getattr(Route, attr.upper())
    return val.replace('<', '{').replace('>', '}')

def main():
    from abyss.server.app import main
    main()



#!/usr/bin/env python

from tinydb import TinyDB, where

DB_PATH = './abyss_db.json'
DB = TinyDB(DB_PATH)

def deserialize(serialized):
    module = __import__(serialized['_module'])
    for modattr in serialized['_module'].split('.')[1:]:
        module = getattr(module, modattr)
    cls = getattr(module, serialized['_type'])
    obj = cls.deserialize(**serialized)
    obj.eid = getattr(serialized, 'eid', None)
    return obj

def serialize(obj):
    return obj._serialize()

class stored(object):

    def _serialize(self):
        data = {
            '_type': self.__class__.__name__,
            '_module': self.__class__.__module__,
        }
        data.update(self.serialize())
        return data

    def serialize(self):
        return {
            attr: getattr(self, attr) 
            for attr in self.__slots__
        }

    @classmethod
    def deserialize(cls, **kwargs):
        return cls(**kwargs)

    def store(self):
        table = DB.table(self.__class__.__name__)
        d = self._serialize()
        if not hasattr(self, 'eid'):
            self.eid = table.insert(d)
        else:
            table.update(d, eids=[self.eid])

class Test(stored):
    __slots__ = ('foo', 'bar')

    def __init__(self, foo=None, bar=None, **kwargs):
        self.foo = foo
        self.bar = bar
